﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using Microsoft.WindowsAPICodePack;
using Microsoft.WindowsAPICodePack.Sensors;
using System.ComponentModel;
using System.Windows.Threading;

namespace Microsoft.WindowsAPICodePack.Sensors.Controls
{
    /// <summary>
    /// Interaction logic for UserControl1.xaml
    /// </summary>
    public partial class AcceleromaterControl : UserControl
    {
        public AcceleromaterControl()
        {
            InitializeComponent();
        }

        public bool AccelermatorEnabled { get; set; }
        public Accelerometer3D Accelerometer3D { get; protected set; }

        public float X { get; protected set; }
        public float Y { get; protected set; }
        public float Z { get; protected set; }

        public delegate void DataChangedEventHandler(object sender, DataChangedEventArgs e);
        public event DataChangedEventHandler DataChanged;

        private static bool? _isInDesignMode;

        /// <summary>
        /// Gets a value indicating whether the control is in design mode (running in Blend
        /// or Visual Studio).
        /// </summary>
        public static bool IsInDesignModeStatic
        {
            get
            {
                if (!_isInDesignMode.HasValue)
                {
                    var prop = DesignerProperties.IsInDesignModeProperty;
                    _isInDesignMode
                        = (bool)DependencyPropertyDescriptor
                        .FromProperty(prop, typeof(FrameworkElement))
                        .Metadata.DefaultValue;
                }

                return _isInDesignMode.Value;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            AccelermatorEnabled = true;

            while (AccelermatorEnabled)
            {
                SensorList<Accelerometer3D> sensorList = null;

                try
                {
                    sensorList = SensorManager.GetSensorsByTypeId<Accelerometer3D>();
                }
                catch (Exception)
                {
                    if (IsInDesignModeStatic)
                        return;

                    if (MessageBox.Show("センサーを接続してください", "", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                        continue;
                }

                if (AccelermatorEnabled)
                {
                    if (sensorList.Count() == 0)
                    {
                        if (IsInDesignModeStatic)
                            return;

                        if (MessageBox.Show("センサーを接続してください", "", MessageBoxButton.OKCancel) == MessageBoxResult.Cancel)
                            break;
                    }
                    else
                    {
                        Accelerometer3D = sensorList.First();
                        break;
                    }
                }
            }

            if (Accelerometer3D != null)
            {
                Accelerometer3D.DataReportChanged += new DataReportChangedEventHandler(Accelerometer3D_DataReportChanged);
                Accelerometer3D.AutoUpdateDataReport = true;
            }
        }

        void Accelerometer3D_DataReportChanged(Sensor sender, EventArgs e)
        {
            try
            {
                X = Accelerometer3D.CurrentAcceleration[AccelerationAxis.X];
                Y = Accelerometer3D.CurrentAcceleration[AccelerationAxis.Y];
                Z = Accelerometer3D.CurrentAcceleration[AccelerationAxis.Z];

                Dispatcher.Invoke(new Action(delegate() { labelAccelerometer.Content = String.Format("X : {0:F2}, Y: {1:F2}, Z : {2:F2}", X, Y, Z); }));

                if (DataChanged != null)
                {
                    Dispatcher.Invoke(new Action(delegate() { DataChanged.Invoke(this, new DataChangedEventArgs(X, Y, Z)); }));
                }
            }
            catch (Exception)
            {
                Accelerometer3D = null;
                Dispatcher.Invoke(new Action(delegate() { labelAccelerometer.Content = "センサーが接続されていません"; }));
            }
        }
    }

    public class DataChangedEventArgs : EventArgs
    {
        public DataChangedEventArgs(float x, float y, float z)
        {
            X = x;
            Y = y;
            Z = z;
        }

        public float X { get; private set; }
        public float Y { get; private set; }
        public float Z { get; private set; }
    }
}
